
/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.018        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: buf.c
 * Program name: library material
 * Source of file: West Midlands OPUS BBS
 * Purpose: General routines for ring buffer manipulation.
 * Changes: <who what when & why major changes have been made>      
 ********************************************************************/

/*
 * BUF.C
 *
 *	Copyright (C) 1984 by Larry Campbell, 73 Concord St., Maynard Mass.
 *
 *	This software may be freely copied and disseminated for
 *	noncommercial purposes, if and only if this entire copyright
 *	statement and notice is included intact.  This software, and
 *	the information contained herein, may not be used for commercial
 *	purposes without my prior written permission.
 *
 *	Implements simple ring buffers
 */

#include <stdio.h>		/* just for definition of NULL */
#include "buf.h"

/*
 * create_buf (size)
 *
 *	Create ring buffer of SIZE characters.  Return NULL if failure,
 *	address of buffer descriptor if successful.
 */

struct ring_buf *create_buf (size)
int size;
{
	struct ring_buf *new_buf;

	if (size < 2)
    		return (NULL);
	if ((new_buf = malloc (sizeof (struct ring_buf))) == NULL)
    		return (NULL);
	if ((new_buf->cdata = malloc (size)) == NULL)
    	{
    		free (new_buf);
    		return (NULL);
    	};
	new_buf->putter = 0;
	new_buf->taker = 0;
	new_buf->size = size;
	return (new_buf);
}

/*
 * destroy_buf (buf)
 *
 *	Release buffer and descriptor
 */

destroy_buf (buf)
struct ring_buf *buf;
{
	free (buf->cdata);
	free (buf);
}

/*
 * put_buf (buf, char)
 *
 *	Put character into buffer.  Return true (1) if successful,
 *	false (0) if buffer full.
 */

int put_buf (buf, c)
struct ring_buf *buf;
char c;
{
	int next;

	if ((next = ((buf->putter + 1) % buf->size)) == buf->taker)
    		return (0);
	buf->cdata[buf->putter] = c;
	buf->putter = next;
	return (1);
}

/*
 * get_buf (buf)
 *
 *	Get character from buffer, return 0 if buffer empty (user should
 *	verify nonemptiness by calling buf_emptyp first)
 */

char get_buf (buf)
struct ring_buf *buf;
{
	char c;

	if (buf->taker == buf->putter)
	    	return (0);
	c = buf->cdata[buf->taker];
	buf->taker = (buf->taker + 1) % buf->size;
	return (c);
}

/*
 * un_get_buf (buf, c)
 *
 *	"unget" a character from a buffer, return true (1) if successful,
 *	false (0) if failure because buffer full or character being
 *	"ungotten" wasn't the same character "gotten"
 */

int un_get_buf (buf, c)
struct ring_buf *buf;
char c;
{
	int dest;

	if ((dest = buf->taker - 1) < 0)
    		dest = buf->size - 1;
	if (buf->putter == dest)
    		return (0);			/* buffer full */
	if (buf->cdata[dest] != c)
    		return (0);			/* caller trying to cheat */
	buf->taker = dest;
	return (1);
}

/*
 * buf_emptyp (buf)
 *
 *	Test for emptiness of a buffer
 */

int buf_emptyp (buf)
struct ring_buf *buf;
{
	return (buf->putter == buf->taker);
}

/*
 * buf_fullp (buf)
 *
 *	Test for fullness of buffer
 */

buf_fullp (buf)
struct ring_buf *buf;
{
	return (((buf->putter + 1) % buf->size) == buf->taker);
}
