/*-------------------------------------------------------*
 * CURSES.H:  Copyright (c) 1987, Allen I. Holub.        *
 *                  All rights reserved.                 *
 *-------------------------------------------------------*
 */

typedef struct
{
    int x_org;     /* X coordinate of upper-left corner    */
    int y_org;     /* Y coordinate of upper-left corner    */
    int x_size;    /* Horizontal size of text area.        */
    int y_size;    /* Vertical   size of text area.        */
    int row;       /* Current cursor row (0 to y_size-1)   */
    int col;       /* Current cursor column (0 to x_size-1)*/
    int scroll_ok; /* Scrolling permitted in this window   */
}
WINDOW;

# define        bool    unsigned int
# define        reg     register
# define        TRUE    (1)
# define        FALSE   (0)
# define        ERR     (0)
# define        OK      (1)

/*------------------------------------------------------------
 * Reminder: The comma opperator goes left to right and
 *              evaluates to the rightmost thing in the list.
 *
 * The following macros implement many of the curses functions
 * note that stdscr only has meaning when passed to getyx.
 *------------------------------------------------------------
 */

#define stdscr          0 

#define getyx(win,y,x)  (win ? ((x)=win->col,(y)=win->row)  \
                             : getpos(&x,&y))

#define mvinch(y,x)     ( move (  y,x), inch ( ) )
#define mvwinch(w,y,x)  ( wmove(w,y,x), winch(w) )

#define addch(c)           putchar(c)
#define endwin()
#define erase()            clear()
#define initscr()
#define printw             printf
#define refresh()
#define scroll(w)          wscroll(w,1);
#define scrollok(win,flag) ((win)->scroll_ok = (flag))
#define subwin(w,a,b,c,d)  newwin(a,b,c,d)
#define wclear             werase
#define wrefresh(win)

/*----------------------------------------------------------*/

extern  WINDOW *newwin  (int ,int ,int ,int );
extern  int box         (WINDOW *,int ,int );
extern  int clear       (void);
extern  int crmode      (void);
extern  int echo        (void);
extern  int getch       (void);
extern  int move        (int ,int );
extern  int nl          (void);
extern  int nocrmode    (void);
extern  int noecho      (void);
extern  int nonl        (void);
extern  int waddch      (WINDOW *, int  );
extern  int waddstr     (WINDOW *, char *);
extern  int wclrtoeol   (WINDOW *       );
extern  int werase      (WINDOW *       );
extern  int wgetch      (WINDOW *       );
extern  int wmove       (WINDOW *, int   ,int );
extern  int wprintw     (WINDOW *, char *,... );
extern  int wscroll     (WINDOW *, int       );


