/*	DOPRNT / FDOPRNT
 *	The actual formatting routines for printf sprintf etc
 *
 *	- #define FLOAT to support the %f conversion (this can be
 *		done from the command line w/ a -dFLOAT option.
 *		In this case, the compiled subroutine will be called
 *		fdoprnt( ) rather than doprnt( ).
 *
 *	- The nonstandard conversion %b (binary) is supported.
 */

extern	char	*ltos(long,	char*,	int);
extern	char	*dtos(double,	char*,	int);

/*------------------------------------------------------------------------------
 *	Macros to save some code space below. If your compiler doesn't
 *	accept multiline macros, remove the backslashes and merge the
 *	entire definition onto one line.
 *
 *	PAD(fw)		outputs filchar, fw times. fw = 0.
 *	TOINT(p,x)	works like "x = atoi(p);" except p
 *				is advanced past the number.
 *
 */

#define	PAD(fw)		while( --fw >= 0 )	\
				(*out)( filchar, o_param )

#define	TOINT(p,x)	while( '0' <= *p && *p <= '9' )	\
				x = (x * 10) + (*p++ - '0')
				
/*------------------------------------------------------------------------------
 *	INTMASK is a portable way to mask off the bottom N bits of a
 *	long, where N is the width of an int in bits.
 */

#define	INTMASK		(long)( (unsigned)(~0) )

/*------------------------------------------------------------------------------
 * Many compilers don't accept this expression:
 *
 *		int	*ap;
 *		x = *( (long *)ap )++;
 *
 * Consequently, I've used the form:
 *
 *		char *ap;
 *		x = *( (long *)ap );
 *		ap += sizeof(long);
 *
 * Less elegant, but more portable.
 */

#define	NEXTARG(x, type)			\
		{				\
			x = *(type *)ap ;	\
			ap += sizeof(type);	\
		}
	
/*------------------------------------------------------------------------------
 *	Use fdoprnt as the subroutine name if FLOAT is defined, else
 *	use doprnt.
 */

#ifdef	FLOAT
#define	DOPRNT	fdoprnt
#else
#define	DOPRNT	doprnt
#endif

/*----------------------------------------------------------------------------*/

char	*DOPRNT	(out, o_param, format, ap)
int	(*out)( );		/* Output subroutine			*/
int	o_param;		/* Second argument to pass to out( )	*/
char	*format;		/* Pointer to format string		*/
char	*ap;			/* Pointer to arguments			*/
{
	char	filchar	;	/* Fill character used to pad fields	*/
	char	nbuf[34];	/* Buffer used to hold converted #s	*/
	char	*bp	;	/* Pointer to current output buffer	*/
	int	slen	;	/* Length of string pointed to by bp	*/
	int	base	;	/* Current base (%x=16, %d=10, etc.)	*/
	int	fldwth	;	/* Field width as in %10x		*/
	int	prec	;	/* Precision as in %10.10s or %10.3f	*/
	int	lftjust	;	/* 1 = left justifying (eg. %-10d)	*/
	int	longf	;	/* Doing long int (%lx or %ld, etc.)	*/
	long	lnum	;	/* Hold numeric arguments		*/
		
#ifdef	FLOAT
	double	dnum	;	/* Used to hold double arguments	*/
#endif

	for(; *format; format++ )
	{
		if( *format != '%')	/* No conversion, just	*/
		{			/* print the next char.	*/
			(*out)(*format, o_param);
		}
		else			/* Process a % conversion	*/
		{
			bp	= nbuf	;
			filchar	= ' '	;
			fldwth	= 0	;
			lftjust	= 0	;
			longf	= 0	;
			prec	= 0	;
			slen	= 0	;
			
			/* Interpret any modifiers that can precede a
			 * conversion character. ( %04... , %-10.6..., etc).
			 * If an * is present instead of a field width then
			 * the width is taken from the argument list.
			 */
			 
			if( *++format == '-') { ++format ; ++lftjust;	}
			if( *format   == '0') { ++format  ; filchar = '0';}
			
		
			if( *format != '*' )
				TOINT( format, fldwth );
			else
			{
				format++;
				NEXTARG( fldwth , int );
			}
			
			if( *format == '.' ) { ++format; TOINT(format, prec);}
			if( *format == 'l' ) { ++format; ++longf;	}
			
		
			/*
			 * By now we've picked off all the modifiers and
			 * *format is looking at the actual conversion
			 * character. Pick the appropriate sized argument
			 * off the stack and advance the pointer (ap) to
			 * point at the next argument.
			 */
			 
			switch( *format )
			{
			default	:	*bp++ = *format	;
					break;
					
			case 'c' :	NEXTARG( *bp++, int );
					break;
					
			case 's' :	NEXTARG( bp, char *);
					break;
					
#ifdef	FLOAT
			case 'f' :
				
				NEXTARG( dnum, double );
				if( dnum < 0 && filchar == '0')
				{
					/* If we are zero filling then we have
					 * to print the - now so that we won't
					 * get "0000-5.2."
					 */
					 
					(*out)( '-', o_param);
					--fldwth;
					dnum = -dnum;
				}
				
				bp = dtos( dnum, bp, prec ? prec : 6 );
				break;
#endif
			case 'd' :	base = 10;
					goto pnum;
					
			case 'x' :	base = 16;
					goto pnum;
				
			case 'b' :	base = 2;
					goto pnum;
				
			case 'o' :	base = 8;
					/* fall through	*/
pnum:
				/* Fetch a long or int-sized argument off the
				 * stack as appropriate. If the fetched number
				 * is not a base 10 int, then mask off the top
				 * bits to prevent sign extension.
				 */
				 
				if( longf )
					NEXTARG( lnum, long )	/* No ;	*/
				else
				{
					NEXTARG( lnum, int );
					if( base != 10 )
					lnum &= INTMASK;
				}
				
				if( lnum < 0L && base == 10 && filchar == '0')
				{
					/* Again, print a - to avoid "000-123."
					 * only decimal numbers get - signs.
					 */
					 
					(*out)( '-', o_param);
					--fldwth;
					lnum = -lnum;
				}
					
				bp = ltos( lnum, bp, base );
				break;
			}
			
			/* Terminate the string if necessary and compute
			 * the string length (slen). Bp will point to the
			 * beginning of the output string.
			 */
			 
			if(*format != 's')
			{
				*bp = '\0';
				slen = bp - nbuf;
				bp = nbuf;
			}
			else
			{
				slen = strlen(bp);
				if( prec && slen > prec )
					slen = prec;
			}
			
		
			/* Adjust fldwth to be the amount of padding we need
			 * to fill the buffer out to the specified field
			 * width. Then print leading padding (if we aren't
			 * left justifying), the buffer itself, and any
			 * required trailing padding (if we are left
			 * justifying).
			 */
			 
			if( (fldwth -= slen) < 0)
				fldwth = 0;
				
			if(!lftjust)
				PAD( fldwth );
				
			while( --slen >= 0 )
				(*out)(*bp++,o_param);
				
			if(lftjust)
				PAD( fldwth );
				
		}	/* end else	*/
	}		/* end for(; *format; format++)	*/
}


/*---------------------------------------------------------------------------*/

#ifdef	DEBUG

#include <stdio.h>

printm (format, args)
char	*format;
char	*args;
{
	/* a version of printf. Call it printm( ) so that we can
	 * use the real printf for debugging.
	 */
	 
	extern int fputc( );
	
	DOPRNT( fputc, stdout, format, &args );
}

/*---------------------------------------------------------------------------*/

main( )
{
/*	printm("%d %x %o %ld %lx %lo %3.1f %c %s\n",
				0, 1, 2, 3L, 4L, 5L, 6.7, '8', "9" );
				
	printm("%s %s %c", "hello", "world", '\n');
	printm("should see <string>  : <%6.6s>\n","string NO NO NO");
	printm("should see <70000>   : <%ld>\n",70000L);
	printm("should see <fffff>   : <%lx>\n",0xfffffL);
	printm("should see <ffff>    : <%x>\n",	-1);
	printm("should see <-1>      : <%ld>\n", -1L);	
	printm("should see <x>       : <%c>\n", 'x');
	printm("should see <a5>      : <%x>\n",0xa5);
	printm("should see <765>     : <%o>\n",0765);
	printm("should see <1010>    : <%b>\n",0xa);
	printm("should see <   123>  : <%6d>\n",123);
	printm("should see <   456>  : <%*d>\n",6,456);
	printm("should see <  -123>  : <%6d>\n",-123);
	printm("should see <123   >  : <%-6d>\n",123);
	printm("should see <-123  >  : <%-6d>\n",-123);
	printm("should see <-00123>  : <%06d>\n",-123);
*/
#ifdef	FLOAT
	printm("should see < 1.234>  : <%6.3f>\n",1.234);
	printm("should see <01.234>  : <%06.3f>\n",1.234);
	printm("should see <1.010 >  : <%-6.3f>\n",1.010);
	printm("should see <-1.3  >  : <%-6.1f>\n",-1.26);
	printm("should see <  -1.2>  : <%6.1f>\n",-1.24);
	printm("should see <1.234567>: <%f>\n",1.234567);
	printm("should see <1>       : <%1.1f>\n",1.234567);
#endif
}
#endif


	
