typedef enum    {BW40,CO40,BW80,CO80,
                 CO320,BW320,BW640} scrmode;
typedef enum    {BLACK,BLUE,GREEN,CYAN,
                 RED,MAGENTA,BROWN,WHITE,
                 GRAY,LBLUE,LGREEN,LCYAN,
                 LRED,LMAGENTA,YELLOW,HIWHITE} colors;
typedef enum    {UP,DOWN} direction;
typedef enum    {BKGD,GGREEN,GRED,GYELLOW,
                 INV,GCYAN,GMAGENTA,GWHITE} Gcolors;
typedef struct {int OTX,OTY,OBX,OBY;
                int CX,CY;
                int TopX,TopY,BotX,BotY;
                int *Buffer;
                void *Previous;
               } window;
#define CGABASE		0xb800
#define MONOBASE	0xb000

/*									*/
/*	SetMode : sets the screen mode and resets the page 		*/
/*	number to zero. allowed screen modes are as follows:		*/
/*									*/
/*	BW40	: Black and white 40*25					*/
/*	CO40	: Color 40*25						*/
/*	BW80	: Black and white 80*25					*/
/*	CO80	: Color 80*25						*/
/*	CO320	: Color 320*200						*/
/*	BW320	: Black and white 320*200				*/
/*	BW640	: Black and white 640*200				*/
/*									*/

void SetMode (scrmode Mode);


/*									*/
/*	SelectScreenPage : This takes a new page and displays it	*/
/*	on screen.							*/
/*									*/

void SelectScreenPage (int NewPage);


/*									*/
/*	GotoXY : Takes two parameters X and Y respectively and		*/
/*	moves the cursor to that position				*/
/*									*/

void GotoXY (int x,int y);


/*									*/
/*	SetTextColor : Takes two parameters the foreground and		*/
/*	background colours these are listed in the typedef colors	*/
/*									*/

void SetTextColor (colors ForeGrnd, colors BackGrnd);

/*									*/
/*	ScrollScreen : This takes the following parameters		*/
/*									*/
/*	Dir	  : UP or DOWN this is the direction of the scroll	*/
/*	NoOfLines : The number of lines to scroll			*/
/*		    if the number is zero it clears the window		*/
/*	TopX	  : The X coordinate of the top left corner		*/
/*	TopY	  : The Y coordinate of the top left corner		*/
/*	BotX	  : The X coordinate of the bottom right corner		*/
/*	BotY	  : The Y coordinate of the bottom right corner		*/

void ScrollScreen (direction Dir, int NoOfLines,int TopX, int TopY,
              int BotX, int BotY);

/*									*/
/*	SetTextBorder : This takes one parameter to set the border	*/
/*	color.								*/
/*									*/

void SetTextBorder (colors Col);

/*									*/
/*	SetGraphicsBackground : This takes one parameter of type 	*/
/*	color and sets the background color.				*/
/*									*/
void SetGraphicsBackground (colors Col);

/*									*/
/*	SetGraphicsPallette : This takes one parameter of type int	*/
/*	as follows:							*/
/*									*/
/*	Pallette No	Colours						*/
/*		0	Green	Red	Brown				*/
/*		1	Cyan 	magenta	LightGray			*/
/*		2	LightGreen	LightRed	Yellow		*/
/*		3	LightCyan	LightMagenta	White		*/
/*									*/
void SetGraphicsPallette (int Pal);

/*									*/
/*	PutChar : Puts a character into the current text window		*/
/*	in the current color						*/
/*									*/
void PutChar (char CH);

/*									*/
/*	Puts : Puts a string in the current text color into		*/
/*	the current text window						*/
/*									*/
void Puts (char far *String);

/*									*/
/*	GPutChar : Puts a character into the current graphics window	*/
/*	in the current color						*/
/*									*/
void GPutChar (char CH);

/*									*/
/*	GPuts : Puts a string in the current Graphics color into	*/
/*	the current graphic window					*/
/*									*/
void GPuts (char far *String);

/*									*/
/*	SetGTextColor : Takes one parameter of type Gcolors		*/
/*									*/
void SetGTextColor (Gcolors col);

/*									*/
/*	Window : Sets up a text window according to the following	*/
/*	parameters:							*/
/*									*/
/*	TopX	  : The X coordinate of the top left corner		*/
/*	TopY	  : The Y coordinate of the top left corner		*/
/*	BotX	  : The X coordinate of the bottom right corner		*/
/*	BotY	  : The Y coordinate of the bottom right corner		*/
/*									*/
void Window (int TopX,int TopY,int BotX,int BotY);

/*									*/
/*	ClearWindow : This clears the current text window to the	*/
/*	current text color						*/
/*									*/
void ClearWindow (void);

/*									*/
/*	Mplot : This takes three parameters the X coordinate		*/
/*	the Y coordinate and the color of the point to be plotted	*/
/*	this plots in the CO320 mode.					*/
/*
									*/
void Mplot (int x,int y, Gcolors col);

/*									*/
/*	GDrawLine : This takes the following parameters :		*/
/*	X1	Start X Coordinate					*/
/*	Y1	Start Y Coordinate					*/
/*	X2	End X Coordinate					*/
/*	Y2	End Y Coordinate					*/
/*	col	Color to plot in.					*/
/*									*/

void GDrawLine (int X1, int Y1, int X2, int Y2, Gcolors col);

/*									*/
/*	HDrawLineFrGr : This takes the following parameters :		*/
/*	X1	Start X Coordinate					*/
/*	Y1	Start Y Coordinate					*/
/*	X2	End X Coordinate					*/
/*	Y2	End Y Coordinate					*/
/*	And Plots in the foreground color in BW640			*/
/*									*/

/*									*/
/*	Hplot : Plots a point at X,Y in BW640				*/
/*									*/

void Hplot (int X, int Y);

/*									*/
/*	Hunplot : UnPlots a point at X,Y in BW640			*/
/*									*/

void Hunplot (int X, int Y);

void HDrawLineFrGr (int X1, int Y1, int X2, int Y2);

/*									*/
/*	HDrawLineBkGr : This takes the following parameters :		*/
/*	X1	Start X Coordinate					*/
/*	Y1	Start Y Coordinate					*/
/*	X2	End X Coordinate					*/
/*	Y2	End Y Coordinate					*/
/*	And Plots in the background color in BW640			*/
/*									*/

void HDrawLineBkGr (int X1, int Y1, int X2, int Y2);

/*									*/
/*	whereisx,whereisy : return back integers values of the 		*/
/*	x & y coordinates of the cursor respectively			*/
/*									*/

int whereisx (void);
int whereisy (void);

/*									*/
/*	CloseWindow : closes down a window and replaces any		*/
/*	text behind the window						*/
/*									*/

int CloseWindow (void);

/*									*/
/*	OpenWindow : Opens a window usual comments about parameters	*/
/*	but adds a title to the top of the window. Saves what 		*/
/*	is there currently and adds the window to the window		*/
/*	"Stack" 							*/
/*	IMPORTANT : Only works in text mode and does no checking	*/
/*	of parameters.							*/
/*									*/

void OpenWindow (int TopX,int TopY, int BotX,int BotY, char * Title);

/*									*/
/*	I finally worked out how to do printf usual comments apply	*/
/*	GPrintf is for graphics, Printf is for text			*/
/*									*/
/*	DO NOT PRINTF MORE THAN 2000 CHARACTERS (one screen full)	*/

void GPrintf(char *format,...);
void Printf(char *format,...);

void DrawLine (int X1,int Y1,int X2,int Y2,Gcolors Col);
