
/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.018        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: strings.c
 * Program name: strings
 * Source of file: Original program Copyright Martin Houston.
 * Purpose: Extracts likeley ASCII strings from a binary file.
 * Mainly useful for finding clues to adventures etc.
 * Changes: <who what when & why major changes have been made>      
 ********************************************************************/

/*
 * Program to identify possible ASCII strings in a binary
 * file. Very usefull for finding clues to adventure games etc.
 *
 * Written in MIX C by Martin Houston.
 * Should work with any C that supports the stdio library.
 */
#include <stdio.h>
#define STRLEN 255 /* Any sequence longer than this will not be
                    * considered as a string
		    */
#define MINLEN 5   /* Any sequence shorter than this will not be
                    * considered as a string
		    */

#ifndef isprint
#define isprint(X) ((X) >= ' ' && (X) <= '~') /* True if x printable ASCII */
#endif

main(argc, argv)
int argc;
char *argv[];
{
	FILE *fp;

	if(argc == 1)
	{
		/* we want to process stdin */
		strscan("stdin", stdin);
        }
	else
	/*
	 * This is a quick & compact way of processing each argument
	 * in turn. The "rb" is to open a binary file under MS-DOS.
	 * Other OSes may need just open for reading.
	 */
	while(argc > 1 && (fp = fopen(argv[1], "rb")) != NULL)
	{
		strscan(argv[1], fp); /* look for strings */
		fclose(fp);
		argv++; /* increment pointer */
		argc--; /* decrement count */
	}
}

/*
 * Print the string formed so far if one is valid.
 */
pstring(curbyte,fname, s)
long curbyte;
char *fname;
char *s;
{
	if((s != NULL) && (s[0] != '\0') && (strlen(s) > MINLEN))
	{
		printf("%s(0x%06lx) :", fname, curbyte);
		printf("%s\n", s);
	}
}
/*
 * Scan for valid strings in the file.
 */
strscan(fname, fp)
char *fname;
FILE *fp;
{
	static char s[STRLEN];	/* the temp string */
	char *sp;	   	/* current position in string */
	int c;		   	/* current character */
	long cnt, strstart;

	sp = s;
	for(strstart = cnt = 0L; (c = getc(fp)) != EOF; cnt++)
	{
		if(c == '\0' || c == '\n' ||
		   c == '\f' || c == '\r' || sp > &(s[STRLEN-1]))
		{
			/* flush string we have built so far if any */
			*sp++ = '\0';
			pstring(strstart, fname, s);
			sp = s;
			strstart = cnt;
		}
		else
		if(isprint(c)|| c == '\t') /* a TAB is considered printable */
		{
			*sp++ = c; /* include in string */
		}
		else
		{
			sp = s; /* reset string as got bad character */
			strstart = cnt; /* remember string start offset */
		}
	}
}

